"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadUserFunction = loadUserFunction;
const path_1 = __importDefault(require("path"));
const read_pkg_up_1 = __importDefault(require("read-pkg-up"));
const url_1 = require("url");
const function_registry_1 = require("./function-registry");
async function isEsModule(modulePath) {
    const ext = path_1.default.extname(modulePath);
    if (ext === '.mjs') {
        return true;
    }
    if (ext === '.cjs') {
        return false;
    }
    const pkg = await (0, read_pkg_up_1.default)({
        cwd: path_1.default.dirname(modulePath),
        normalize: false
    });
    // If package.json specifies type as 'module', it's an ES module.
    return pkg?.packageJson.type === 'module';
}
const dynamicImport = new Function('modulePath', 'return import(modulePath)'
// eslint-disable-next-line @typescript-eslint/no-explicit-any
);
async function loadUserFunction(codeLocation, functionTarget) {
    try {
        const functionModulePath = getFunctionModulePath(codeLocation);
        if (functionModulePath === null) {
            return {
                reason: `Provided code location '${codeLocation}' is not a loadable module.`
            };
        }
        let functionModule;
        const esModule = await isEsModule(functionModulePath);
        if (esModule) {
            // Resolve module path to file:// URL. Required for windows support.
            const fpath = (0, url_1.pathToFileURL)(functionModulePath);
            functionModule = await dynamicImport(fpath.href);
        }
        else {
            functionModule = require(functionModulePath);
        }
        // If the customer declaratively registered a function matching the target
        // return that.
        const registeredFunction = (0, function_registry_1.getRegisteredFunction)(functionTarget);
        if (registeredFunction) {
            return registeredFunction;
        }
        const userFunction = functionTarget
            .split('.')
            .reduce((code, functionTargetPart) => {
            if (typeof code === 'undefined') {
                return undefined;
            }
            else {
                return code[functionTargetPart];
            }
        }, functionModule);
        if (typeof userFunction === 'undefined') {
            return {
                reason: `Function '${functionTarget}' is not defined in the provided module.`
            };
        }
        if (typeof userFunction !== 'function') {
            return {
                reason: `'${functionTarget}' needs to be of type function. Got: ${typeof userFunction}`
            };
        }
        return { userFunction: userFunction };
    }
    catch (ex) {
        const err = ex;
        let additionalHint;
        if (isNodeError(err) && err.code === 'MODULE_NOT_FOUND') {
            additionalHint =
                'Did you list all required modules in the package.json dependencies?\n';
        }
        else {
            additionalHint = 'Is there a syntax error in your code?\n';
        }
        return {
            reason: `Provided module can't be loaded.\n${additionalHint}` +
                `Detailed stack trace: ${err.stack}`
        };
    }
}
/**
* Returns resolved path to the module containing the user function.
* Returns null if the module can not be identified.
* @param codeLocation - Directory with user's code
* @return Resolved path or null.
*/
function getFunctionModulePath(codeLocation) {
    try {
        return require.resolve(codeLocation);
    }
    catch (ex) {
        // Ignore exception, this means the function was not found here.
        // textLogger.error('Failed to resolve code location.', ex)
    }
    return null;
}
const isNodeError = (error) => error instanceof Error;
//# sourceMappingURL=function-loader.js.map