"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTextTypeForTesting = exports.accessLogMiddleware = void 0;
const os_1 = require("os");
const whatwg_mimetype_1 = __importDefault(require("whatwg-mimetype"));
const async_context_1 = require("../async-context");
const constants_1 = require("../constants");
const error_1 = require("../error");
const logger_1 = require("../logger");
var LogType;
(function (LogType) {
    LogType["ACCESSLOG"] = "accesslog";
    LogType["USERCODELOG"] = "usercodelog";
})(LogType || (LogType = {}));
const accessLogMiddleware = (option) => {
    const fixedBaseInfo = getFixedBaseLogInfo();
    return async (ctx, next) => {
        const start = new Date();
        await next();
        const end = new Date();
        const { logHeaderBody, logEventContext } = option;
        const tcbContext = ctx.state.tcbContext;
        const wechatContext = ctx.state.wechatContext;
        const errorResult = ctx.state.errorResult;
        const eventID = (0, async_context_1.getCurrentEventID)();
        const reqType = ctx.headers['content-type'] || '';
        const resType = ctx.response.type;
        const log = {
            '@timestamp': start.toISOString(),
            startAt: start.toISOString(),
            endAt: end.toISOString(),
            logType: LogType.ACCESSLOG,
            ...fixedBaseInfo,
            eventId: eventID,
            method: ctx.method,
            url: ctx.href,
            path: ctx.path,
            status: ctx.status,
            httphost: ctx.headers.host,
            referrer: ctx.headers.referer,
            origin: ctx.headers.origin,
            ua: ctx.get('user-agent'),
            connection: ctx.headers.connection,
            contentType: reqType,
            clientIp: ctx.request.ip,
            reqSize: ctx.request.length,
            resSize: ctx.response.length,
            resContentType: resType,
            timeCost: end.valueOf() - start.valueOf(),
            userTimeCost: ctx.state.userTimeCost,
            code: errorResult?.code,
            message: errorResult?.message,
            ...tcbContext,
            ...wechatContext
        };
        if (logHeaderBody) {
            log.reqHeader = JSON.stringify(ctx.headers);
            log.resHeader = JSON.stringify(ctx.response.headers);
            log.reqBody = isTextType(reqType) ? JSON.stringify(ctx.request.body)?.slice(0, constants_1.LOG_MAX_BODY_LENGTH) : '';
            log.resBody = isTextType(resType) ? JSON.stringify(ctx.response.body)?.slice(0, constants_1.LOG_MAX_BODY_LENGTH) : '';
        }
        if (logEventContext) {
            log.event = ctx.state.eventString;
            log.context = ctx.state.contextString;
        }
        if (ctx.state.error) {
            const error = ctx.state.error;
            log.error = error.message;
            log.stack = (0, error_1.decorateErrorStack)(error);
            log.stackFull = error?.stack;
        }
        logger_1.functionsLogger.logAccesslog('info', JSON.stringify(log));
        logger_1.functionsLogger.logUserCodelog();
    };
};
exports.accessLogMiddleware = accessLogMiddleware;
// https://mimesniff.spec.whatwg.org/#mime-type-groups
const isTextType = (declaration) => {
    const type = whatwg_mimetype_1.default.parse(declaration);
    if (!type) {
        return false;
    }
    if (type.isHTML() || type.isXML() || type.isJavaScript()) {
        return true;
    }
    if (type.subtype.endsWith('+json') || ['application/json', 'text/json'].includes(type.essence)) {
        return true;
    }
    if (type.type === 'text') {
        return true;
    }
    if (type.subtype === 'x-www-form-urlencoded') {
        return true;
    }
    return false;
};
const getFixedBaseLogInfo = () => {
    return {
        hostname: (0, os_1.hostname)(),
        pid: process.pid,
        version: constants_1.SDKVersion,
        nodeEnv: getPrefixedEnv('NODE_'),
        tcbEnv: getPrefixedEnv('TCB_')
    };
};
const getPrefixedEnv = (prefix) => {
    return Object.entries(process.env).filter(([key]) => key.startsWith(prefix))
        .map(([key, value]) => `${key}=${value}`).join(', ');
};
/**
 * This should only be accessed in test files
 */
exports.isTextTypeForTesting = isTextType;
//# sourceMappingURL=accesslog.js.map